@@ -75,6 +75,7 @@ from telemeta.util.logger import Logger
 from telemeta.util.unicode import UnicodeWriter
 from telemeta.cache import TelemetaCache
 import telemeta.web.pages as pages
+from telemeta.forms import *
 
 
 def render(request, template, data = None, mimetype = None):
@@ -286,6 +287,9 @@ class GeneralView(object):
                 collections.by_publish_year(int(value), int(input.get('pub_year_to', value))), 
                 items.by_publish_year(int(value), int(input.get('pub_year_to', value)))),
             'pub_year_to': lambda value: (collections, items),
+            'sound': lambda value: (
+                collections.sound(),
+                items.sound()),
         }
        
         for key, value in input.items():
@@ -343,6 +347,8 @@ class CollectionView(object):
 
     def collection_detail(self, request, public_id, template='telemeta/collection_detail.html'):
         collection = MediaCollection.objects.get(public_id=public_id)
+        items = collection.items.enriched()
+        items = items.order_by('code', 'old_code')
         
         if collection.public_access == 'none' and not (request.user.is_staff or request.user.is_superuser):
             mess = ugettext('Access not allowed') 
@@ -354,7 +360,25 @@ class CollectionView(object):
         public_access = get_public_access(collection.public_access, collection.recorded_from_year, 
                                                 collection.recorded_to_year)
         playlists = get_playlists(request)
-        return render(request, template, {'collection': collection, 'playlists': playlists, 'public_access': public_access})
+        
+        related_media = MediaCollectionRelated.objects.filter(collection=collection)
+        for media in related_media:
+            if not media.mime_type:
+                media.set_mime_type()
+                media.save()
+            if not media.title and media.url:
+                try:
+                    from lxml import etree
+                    parser = etree.HTMLParser()
+                    tree = etree.parse(media.url, parser)
+                    title = tree.find(".//title").text
+                    title = title.replace('\n', '').strip()
+                    media.title = title
+                except:
+                    media.title = media.url
+                media.save()
+                
+        return render(request, template, {'collection': collection, 'playlists': playlists, 'public_access': public_access, 'items': items, 'related_media': related_media})
 
     @method_decorator(permission_required('telemeta.change_mediacollection'))
     def collection_edit(self, request, public_id, template='telemeta/collection_edit.html'):
@@ -393,14 +417,14 @@ class CollectionView(object):
     @method_decorator(permission_required('telemeta.add_mediacollection'))
     def collection_copy(self, request, public_id, template='telemeta/collection_edit.html'):
         if request.method == 'POST':
-            new_collection = MediaCollection()
-            form = MediaCollectionForm(data=request.POST, files=request.FILES, instance=new_collection)
+            collection = MediaCollection()
+            form = MediaCollectionForm(data=request.POST, files=request.FILES, instance=collection)
             if form.is_valid():
                 code = form.cleaned_data['code']
                 if not code:
                     code = public_id
                 form.save()
-                new_collection.set_revision(request.user)
+                collection.set_revision(request.user)
                 return HttpResponseRedirect('/collections/'+code)
         else:
             collection = MediaCollection.objects.get(public_id=public_id)
@@ -425,6 +449,27 @@ class CollectionView(object):
         collection.delete()
         return HttpResponseRedirect('/collections/')
     
+    def related_media_collection_stream(self, request, collection_public_id, media_id):
+        collection = MediaCollection.objects.get(public_id=collection_public_id)
+        media = MediaCollectionRelated.objects.get(collection=collection, id=media_id)
+        response = HttpResponse(stream_from_file(media.file.path), mimetype=media.mime_type)
+#        response['Content-Disposition'] = 'attachment'
+        return response
+    
+    @method_decorator(permission_required('telemeta.change_mediacollection'))
+    def related_media_edit(self, request, public_id, template):
+        collection = MediaCollection.objects.get(public_id=public_id)
+        MediaCollectionRelatedFormSet = inlineformset_factory(MediaCollection, MediaCollectionRelated, form=MediaCollectionRelatedForm)
+        if request.method == 'POST':
+            formset = MediaCollectionRelatedFormSet(data=request.POST, files=request.FILES, instance=collection)
+            if formset.is_valid():
+                formset.save()
+                collection.set_revision(request.user)
+                return HttpResponseRedirect('/collections/'+public_id)
+        else:
+            formset = MediaCollectionRelatedFormSet(instance=collection)
+        
+        return render(request, template, {'collection': collection, 'formset': formset,})
 
 class ItemView(object):
     """Provide Collections web UI methods"""
@@ -440,10 +485,11 @@ class ItemView(object):
         # Get previous and next items
         pks = []
         items = MediaItem.objects.filter(collection=item.collection)
+        items = items.order_by('code', 'old_code')
+        
         if len(items) > 1:
             for it in items:
                 pks.append(it.pk)
-            pks.sort()
             for pk in pks:
                 if pk == item.pk:
                     if pk == pks[0]:
@@ -490,7 +536,9 @@ class ItemView(object):
         # Get TimeSide processors
         formats = []
         for encoder in self.encoders:
-            formats.append({'name': encoder.format(), 'extension': encoder.file_extension()})
+            #FIXME: timeside cannot encode to FLAC and OGG now :'(
+            if encoder.file_extension() != 'ogg' and encoder.file_extension() != 'flac':
+                formats.append({'name': encoder.format(), 'extension': encoder.file_extension()})
 
         graphers = []
         for grapher in self.graphers:
@@ -505,6 +553,23 @@ class ItemView(object):
         playlists = get_playlists(request)
         public_access = get_public_access(item.public_access, str(item.recorded_from_date).split('-')[0], 
                                                 str(item.recorded_to_date).split('-')[0])
+        
+        related_media = MediaItemRelated.objects.filter(item=item)
+        for media in related_media:
+            if not media.mime_type:
+                media.set_mime_type()
+                media.save()
+            if not media.title and media.url:
+                try:
+                    from lxml import etree
+                    parser = etree.HTMLParser()
+                    tree = etree.parse(media.url, parser)
+                    title = tree.find(".//title").text
+                    title = title.replace('\n', '').strip()
+                    media.title = title
+                except:
+                    media.title = media.url
+                media.save()
                 
         return render(request, template,
                     {'item': item, 'export_formats': formats,
@@ -512,6 +577,7 @@ class ItemView(object):
                     'audio_export_enabled': getattr(settings, 'TELEMETA_DOWNLOAD_ENABLED', True),
                     'previous' : previous, 'next' : next, 'marker': marker_id, 'playlists' : playlists, 
                     'public_access': public_access, 'width': width, 'height': height, 
+                    'related_media': related_media, 
                     })
         
     @method_decorator(permission_required('telemeta.change_mediaitem'))
@@ -521,7 +587,9 @@ class ItemView(object):
         
         formats = []
         for encoder in self.encoders:
-            formats.append({'name': encoder.format(), 'extension': encoder.file_extension()})
+            #FIXME: timeside cannot encode to FLAC and OGG now :'(
+            if encoder.file_extension() != 'ogg' and encoder.file_extension() != 'flac':
+                formats.append({'name': encoder.format(), 'extension': encoder.file_extension()})
 
         graphers = []
         for grapher in self.graphers:
@@ -561,6 +629,28 @@ class ItemView(object):
                     'audio_export_enabled': getattr(settings, 'TELEMETA_DOWNLOAD_ENABLED', True), "form": form, 
                     'previous' : previous, 'next' : next, 
                     })
+    
+    def related_media_item_stream(self, request, item_public_id, media_id):
+        item = MediaItem.objects.get(public_id=item_public_id)
+        media = MediaItemRelated.objects.get(item=item, id=media_id)
+        response = HttpResponse(stream_from_file(media.file.path), mimetype=media.mime_type)
+#        response['Content-Disposition'] = 'attachment'
+        return response
+    
+    @method_decorator(permission_required('telemeta.change_mediaitem'))
+    def related_media_edit(self, request, public_id, template):
+        item = MediaItem.objects.get(public_id=public_id)
+        MediaItemRelatedFormSet = inlineformset_factory(MediaItem, MediaItemRelated, form=MediaItemRelatedForm)
+        if request.method == 'POST':
+            formset = MediaItemRelatedFormSet(data=request.POST, files=request.FILES, instance=item)
+            if formset.is_valid():
+                formset.save()
+                item.set_revision(request.user)
+                return HttpResponseRedirect('/items/'+public_id)
+        else:
+            formset = MediaItemRelatedFormSet(instance=item)
+        
+        return render(request, template, {'item': item, 'formset': formset,})
         
     @method_decorator(permission_required('telemeta.add_mediaitem'))
     def item_add(self, request, public_id=None, template='telemeta/mediaitem_add.html'):
@@ -588,14 +678,30 @@ class ItemView(object):
     def item_copy(self, request, public_id, template='telemeta/mediaitem_copy.html'):
         """Copy a given item"""        
         if request.method == 'POST':
-            new_item = MediaItem()
-            form = MediaItemForm(data=request.POST, files=request.FILES, instance=new_item)
+            source_item = MediaItem.objects.get(public_id=public_id)
+            item = MediaItem()
+            form = MediaItemForm(data=request.POST, files=request.FILES, instance=item)
             if form.is_valid():
                 form.save()
                 code = form.cleaned_data['code']
                 if not code:
-                    code = str(new_item.id)
-                new_item.set_revision(request.user)
+                    code = str(item.id)
+                
+                performances = MediaItemPerformance.objects.filter(media_item=source_item)
+                for performance in performances:
+                    performance.pk = None
+                    performance.id = None
+                    performance.media_item = item
+                    performance.save()
+                    
+                keywords = MediaItemKeyword.objects.filter(item=source_item)
+                for keyword in keywords:
+                    keyword.pk = None
+                    keyword.id = None
+                    keyword.item = item
+                    keyword.save()
+
+                item.set_revision(request.user)
                 return HttpResponseRedirect('/items/'+code)
         else:
             item = MediaItem.objects.get(public_id=public_id)
@@ -710,7 +816,7 @@ class ItemView(object):
                 
         response = HttpResponse(self.cache_data.read_stream_bin(image_file), mimetype=mime_type)
         return response
-
+        
     def list_export_extensions(self):
         "Return the recognized item export file extensions, as a list"
         list = []
@@ -762,14 +868,18 @@ class ItemView(object):
             decoder = timeside.decoder.FileDecoder(audio)
             format = decoder.format()
         
+        dc_metadata = dublincore.express_item(item).to_list()
+        mapping = DublinCoreToFormatMetadata(extension)
+        metadata = mapping.get_metadata(dc_metadata)     
+        
         if mime_type in format:
             # source > stream
+            if not extension in mapping.unavailable_extensions:
+                proc = encoder(audio)
+                proc.set_metadata(metadata)
+                proc.write_metadata()
             response = HttpResponse(stream_from_file(audio), mimetype = mime_type)
-            
         else:
-            dc_metadata = dublincore.express_item(item).to_list()
-            mapping = DublinCoreToFormatMetadata(extension)
-            metadata = mapping.get_metadata(dc_metadata)     
             media = self.cache_export.dir + os.sep + file
             if not self.cache_export.exists(file) or flag.value == False:
                 # source > encoder > stream
@@ -781,9 +891,10 @@ class ItemView(object):
                 response = HttpResponse(stream_from_processor(decoder, proc, flag), mimetype = mime_type)
             else:
                 # cache > stream
-                proc = encoder(media)
-                proc.set_metadata(metadata)
-                proc.write_metadata()
+                if not extension in mapping.unavailable_extensions:
+                    proc = encoder(media)
+                    proc.set_metadata(metadata)
+                    proc.write_metadata()
                 response = HttpResponse(self.cache_export.read_stream_bin(file), mimetype = mime_type)
         
         response['Content-Disposition'] = 'attachment'
